/*
 * Calcurse - text-based organizer
 * Copyright (c) 2004-2005 Frederic Culot
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Send your feedback or comments to : calcurse@culot.org
 * Calcurse home page : http://culot.org/calcurse
 *
 */

#include <ncurses.h>	
#include <time.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>

#include "io.h"
#include "help.h"
#include "calendar.h"
#include "utils.h"
#include "vars.h"
#include "event.h"
#include "todo.h"
#include "args.h"


/* Variables for calendar */
struct tm *ptrtime;
time_t timer;
char current_day[3];
char current_month[3];
char current_year[5];
char current_time[15];
char cal_date[30];
int year, month, day;
int sel_year, sel_month, sel_day;

/* Variables for appointments */
int show_event = 0;
int number_events_inday;
int first_app_onscreen = 1;
char event_start_time[100];
char event_end_time[100];
int hilt_app = 0, sav_hilt_app;

/* Variables for todo list */
int nb_tod = 0, hilt_tod = 0, sav_hilt_tod;
int first_todo_onscreen = 1;

/* Variables for user configuration */
int colr = 1, layout = 1;
int no_data_file = 1;
char confirm_quit[4] = "yes";
char confirm_delete[4] = "yes";
char auto_save[4] = "yes";
char skip_system_dialogs[4] = "no";
char really_quit[4] = "ini";

/* 
 * Variables to handle calcurse windows 
 */ 
//WINDOW *awin = NULL, *cwin = NULL, *twin = NULL, *swin = NULL;
int x_cal, y_cal, x_app, y_app, x_tod, y_tod, x_bar, y_bar;
int nl_cal, nc_cal, nl_app, nc_app, nl_tod, nc_tod, nl_bar, nc_bar;
int CALENDAR = 0, APPOINTMENT = 1, TODO = 2;
int which_pan = 0;

/* External functions */
void get_date();
void init_wins(), reinit_wins();
void win_show(WINDOW * win, char *label, int label_color);
void add_todo();
void add_apts();
void load_conf();
void update_todo_panel();
void update_app_panel();
void extract_data(char *dst_data, const char *org, int len);
void get_screen_config();
void update_windows(int surrounded_window);
void general_config();
void print_general_options(WINDOW *win);
void switch_option(int option, char *);
void print_option_incolor(WINDOW *win, char *option, int pos_x, int pos_y);
void del_event();

/*
 * Calcurse  is  a text-based personal organizer which helps keeping track
 * of events and everyday tasks. It contains a calendar, a 'todo' list,
 * and puts your appointments in order. The user interface is configurable,
 * and one can choose between different color schemes and layouts. 
 * All of the commands are documented within an online help system.
 */
int main(int argc, char **argv)
{
	int ch;
	int i;
	int non_interactive;

	/* 
	 * Begin by parsing and handling command line arguments.
	 * The data path is also initialized here.
	 */
	non_interactive = parse_args(argc, argv, colr);
	if (non_interactive) return 0;

	/* Begin of interactive mode with ncurses interface. */
	initscr();		/* start the curses mode */
	cbreak();		/* control chars generate a signal */
	noecho();		/* controls echoing of typed chars */
	curs_set(0);		/* make cursor invisible */
        get_date();
	get_screen_config();
        init_wins();
	
	if (has_colors()) {
		start_color();

		/* Color assignment */
		init_pair(1, COLOR_RED, COLOR_BLACK);
		init_pair(2, COLOR_GREEN, COLOR_BLACK);
		init_pair(3, COLOR_BLUE, COLOR_BLACK);
		init_pair(4, COLOR_CYAN, COLOR_BLACK);
		init_pair(5, COLOR_YELLOW, COLOR_BLUE);
		init_pair(6, COLOR_BLACK, COLOR_GREEN);
		init_pair(7, COLOR_BLACK, COLOR_BLUE);
		init_pair(8, COLOR_RED, COLOR_BLUE);
		init_pair(9, COLOR_WHITE, COLOR_BLACK);
	}

	/* 
	 * Read the data from files : first the user
	 * configuration (the display is then updated), and then
	 * the todo list and appointments.
	 */
	update_windows(which_pan);
	no_data_file = check_data_files();
	load_conf();
	nb_tod = load_todo(colr);	
	load_app();	
	get_screen_config();
        reinit_wins();
        startup_screen(skip_system_dialogs, no_data_file, colr);
	update_windows(CALENDAR);

	/* User input */
	for (;;) {
		
		/* Check terminal size. */
		getmaxyx(stdscr, row, col);
		if ((col < 80) | (row < 24)) {
		        endwin();
			fputs("Please resize your terminal screen\n", stderr);
			fputs("(to at least 80x24)\n", stderr);
			fputs("and restart CalCurse\n", stderr);
			return 1;
		}
		/* Get user input. */
		ch = wgetch(swin);
		switch (ch) {

		case 9:	/* The TAB key was hit. */
			/* Save previously highlighted event. */
			if (which_pan == 2) {
				sav_hilt_tod = hilt_tod;
				hilt_tod = 0;
			}
			if (which_pan == 1) {
				sav_hilt_app = hilt_app;
				hilt_app = 0;
			}
			/* Switch to the selected panel. */
			if (which_pan == 2) which_pan = 0;
			else ++which_pan;

			/* Select the event to highlight. */
			if (which_pan == 1) {
				if ((sav_hilt_app == 0) 
					& (number_events_inday != 0))
					hilt_app = 1;
				else
					hilt_app = sav_hilt_app;
			} else if (which_pan == 2) {
				if ((sav_hilt_tod == 0) & (nb_tod != 0))
					hilt_tod = 1;
				else
					hilt_tod = sav_hilt_tod;
			}
			break;

		case 'R':
		case 'r':	
                        reinit_wins();
			break;

		case 'G':
		case 'g':	/* Goto function */
			erase_window_part(swin, 0, 0, nc_bar, nl_bar);
			get_date();
			goto_day(colr, day, month, year,
				 &sel_day, &sel_month, &sel_year);
			break;

		case 'V':
		case 'v':	/* View function */
			if ((which_pan == 1) & (hilt_app != 0)) {
				show_event = 1;
				update_app_panel();
				show_event = 0;
			}
			if ((which_pan == 2) & (hilt_tod != 0)) {
				show_event = 1;
				update_todo_panel();
				show_event = 0;
			}
			break;

		case 'C':
		case 'c':	/* Configuration menu */
			erase_window_part(swin, 0, 0, nc_bar, nl_bar);
			config_bar(colr);
			while ((ch = wgetch(swin)) != 'q') {
				switch (ch) {
				case 'C':
				case 'c':
					colr = color_config(colr);
					break;
				case 'L':
				case 'l':
					layout = layout_config(layout, colr);
					break;
				case 'G':
				case 'g':
					general_config();
					break;
				}
                                reinit_wins();
				erase_window_part(swin, 0, 0, nc_bar, nl_bar);
				config_bar(colr);
			}
                        update_windows(which_pan);
			break;

		case 'A':
		case 'a':	/* Add an item */
			if (which_pan == APPOINTMENT) add_apts();
			if (which_pan == TODO) add_todo();
			break;

		case 'D':
		case 'd':	/* Delete an item */
			del_event();
			break;

		case '?':	/* Online help system */
			status_bar(which_pan, colr, nc_bar, nl_bar);
			help_screen(which_pan, colr);
			break;

		case 'S':
		case 's':	/* Save function */
			save_cal(auto_save, confirm_quit,
				 confirm_delete, skip_system_dialogs,
                                 colr, layout);
			break;

		case (261):	/* right arrow */
		case ('L'):
		case ('l'):
			if (which_pan == 0) {
				if ((sel_day == 31) & (sel_month == 12))
				{ /* goto next year */
					sel_day = 0;
					sel_month = 1;
					sel_year++;
				}
				if (sel_day == days[sel_month - 1])
				{ /* goto next month */
					sel_month = sel_month + 1;
					sel_day = 1;
				} else
					sel_day = sel_day + 1;
			}
			break;

		case (260):	/* left arrow */
		case ('H'):
		case ('h'):
			if (which_pan == 0) {
				if ((sel_day == 1) & (sel_month == 1))
				{ /* goto previous year */
					sel_day = 32;
					sel_month = 12;
					sel_year--;
				}
				if (sel_day == 1)
				{ /* goto previous month */
					sel_day = days[sel_month - 2];
					sel_month = sel_month - 1;
				} else
					sel_day = sel_day - 1;
			}
			break;

		case (259):	/* up arrow */
		case ('K'):
		case ('k'):
			if (which_pan == 0) {
				if ((sel_day <= 7) & (sel_month == 1))
				{ /* goto previous year */
					sel_day = 31 - (7 - sel_day);
					sel_month = 12;
					sel_year--;
					break;
				}
				if (sel_day <= 7)
				{ /* goto previous month */
					sel_day = days[sel_month - 2] -
					    	  (7 - sel_day);
					sel_month = sel_month - 1;
				} else /* previous week */
					sel_day = sel_day - 7;
			} else {
				if ((which_pan == 1) & (hilt_app > 1)) {
					--hilt_app;
					if (hilt_app < first_app_onscreen)
						--first_app_onscreen;
				}
				if ((which_pan == 2) & (hilt_tod > 1)) {
					--hilt_tod;
					if (hilt_tod < first_todo_onscreen)
						--first_todo_onscreen;
				}
			}
			break;

		case (258):	/* down arrow */
		case ('J'):
		case ('j'):
			if (which_pan == 0) {
				if ((sel_day >= days[sel_month - 1] - 7) & 
				   (sel_month == 12))
				{ /* next year */
					sel_day = (7 - (31 - sel_day));
					sel_month = 1;
					sel_year++;
					break;
				}
				if (sel_day > days[sel_month - 1] - 7)
				{ /* next month */
					sel_day = (7 - (days[sel_month - 1] -
					      	   sel_day));
					sel_month = sel_month + 1;
				} else /* next week */
					sel_day = sel_day + 7;
			} else {
				if ((which_pan == 1) & (hilt_app <
							number_events_inday))
				{
					++hilt_app;
					if (hilt_app - first_app_onscreen ==
					    floor((nl_app - 4) / 3))
						++first_app_onscreen;
				}
				if ((which_pan == 2) & (hilt_tod < nb_tod)) {
					++hilt_tod;
					if (hilt_tod - first_todo_onscreen ==
					    nl_tod - 4)
						++first_todo_onscreen;
				}
			}
			break;

		case ('Q'):	/* Quit calcurse :-( */
		case ('q'):
			if (strncmp(auto_save, "yes", 3) == 0)
				save_cal(auto_save,confirm_quit,
					 confirm_delete, skip_system_dialogs,
					 colr, layout);

			if (strncmp(confirm_quit, "yes", 3) == 0) {
				while ((strncmp(really_quit, "yes",3) !=
					0) & (strncmp(really_quit,"no",
						      3) != 0)) {
				  status_mesg("Do you really want to quit ? [yes or no]", "", colr); 
					getstring(swin, colr, really_quit, 0, 1);
				}
				if (strncmp(really_quit, "yes", 3) == 0) {
					endwin();
					return 0;
				} else {
					erase_window_part(swin, 0, 0, nc_bar, nl_bar);
					strcpy(really_quit, "ini");
					break;
				}
			} else {
				endwin();
				return 0;
			}
			break;

		}	/* end case statement */
		update_windows(which_pan);
	}
}	/* end of interactive mode */

/* 
 * EXTERNAL FUNCTIONS
 */

/* 
 * Update all of the three windows and put a border around the
 * selected window.
 */
void update_windows(int surrounded_window)
{
	update_app_panel();	
	update_todo_panel();
	update_cal_panel(cwin, nl_cal, nc_cal, sel_month,
			 sel_year, sel_day, day, month, year);
	status_bar(surrounded_window, colr, nc_bar, nl_bar);
	if (surrounded_window == CALENDAR) {	
		border_color(cwin, colr);
		border_color(awin, 9);
		border_color(twin, 9);
	} else if (surrounded_window == APPOINTMENT) {
		border_color(awin, colr);
		border_color(cwin, 9);
		border_color(twin, 9);
	} else if (surrounded_window == TODO) {
		border_color(twin, colr);
		border_color(awin, 9);
		border_color(cwin, 9);
	} else { 
		/* NOTREACHED */
		fputs("No window selected in update_windows()\n",stderr);
		exit(1);
	}
        wmove(swin, 0, 0);
	doupdate();
}

/* 
 * Get the screen size and recalculate the windows configurations.
 */
void get_screen_config()
{
	/* Get the screen configuration */
	getmaxyx(stdscr, row, col);

	/* window size definition */
	nl_cal = 12;
	nc_cal = 30;
	nc_app = col - nc_cal;
	nl_app = row - 2;
	nc_tod = nc_cal;
	nl_tod = row - (nl_cal + 2);
	nl_app = row - 2;
	nl_bar = 2; y_bar = row - 2;
	nc_bar = col; x_bar = 0;

	/* defining the layout */
	switch (layout) {
	case 1:
		y_app = 0; x_app = 0; y_cal = 0;
		x_tod = nc_app; y_tod = nl_cal; x_cal = nc_app;
		break;
	case 2:
		y_app = 0; x_app = 0; y_tod = 0;
		x_tod = nc_app; x_cal = nc_app; y_cal = nl_tod;
		break;
	case 3:
		y_app = 0; x_tod = 0; x_cal = 0; y_cal = 0;
		x_app = nc_cal; y_tod = nl_cal;
		break;
	case 4:
		y_app = 0; x_tod = 0; y_tod = 0; x_cal = 0;
		x_app = nc_cal; y_cal = nl_tod;
		break;
	}
}



/* Get current date */
void get_date()
{
	timer = time(NULL);
	ptrtime = localtime(&timer);
	strftime(current_time, 15, "%H:%M%p", ptrtime);
	strftime(cal_date, 30, "%a %B %Y", ptrtime);
	strftime(current_day, 3, "%d", ptrtime);
	strftime(current_month, 3, "%m", ptrtime);
	strftime(current_year, 5, "%Y", ptrtime);
	month = atoi(current_month);
	day = atoi(current_day);
	year = atoi(current_year);
	sel_year = year;
	sel_month = month;
	sel_day = day;
}

/* Create all the windows */
void init_wins()
{
	char label[80];

	cwin = newwin(nl_cal, nc_cal, y_cal, x_cal);
	sprintf(label, "Calendar");
	win_show(cwin, label, colr);
	awin = newwin(nl_app, nc_app, y_app, x_app);
	sprintf(label, "Appointments");
	win_show(awin, label, colr);
	twin = newwin(nl_tod, nc_tod, y_tod, x_tod);
	sprintf(label, "ToDo");
	win_show(twin, label, colr);
	swin = newwin(nl_bar, nc_bar, y_bar, x_bar);
        /* Enable function keys (i.e. arrow keys) in those windows */
        keypad(swin, TRUE);
        keypad(twin, TRUE);
        keypad(awin, TRUE);
        keypad(cwin, TRUE);
}

/* 
 * Delete the existing windows and recreate them with their new
 * size and placement.
 */
void reinit_wins()
{
        clear();
        delwin(swin);
        delwin(cwin);
        delwin(awin);
        delwin(twin);
        get_screen_config();
        init_wins();
        update_windows(which_pan);
}

/* Show the window with a border and a label */
void win_show(WINDOW * win, char *label, int label_color)
{
	int startx, starty, height, width;

	getbegyx(win, starty, startx);
	getmaxyx(win, height, width);

	box(win, 0, 0);
	mvwaddch(win, 2, 0, ACS_LTEE);
	mvwhline(win, 2, 1, ACS_HLINE, width - 2);
	mvwaddch(win, 2, width - 1, ACS_RTEE);

	print_in_middle(win, 1, 0, width, label, COLOR_PAIR(label_color));
}

/* General configuration */
void general_config()
{
	WINDOW *conf_win;
	char label[80];
	char number_str[] = "Enter an option number to change its value [Q to quit] ";
	int ch;

	clear();
	conf_win = newwin(row - 2, col, 0, 0);
	box(conf_win, 0, 0);
	sprintf(label, "CalCurse %s | general options", VERSION);
	win_show(conf_win, label, colr);
	status_mesg(number_str, "",colr);
	print_general_options(conf_win);
	while ((ch = wgetch(swin)) != 'q') {
		switch (ch) {
		case '1':	// auto-save
			switch_option(1, auto_save);
			break;
		case '2':	// confirm-quit
			switch_option(2, confirm_quit);
			break;
		case '3':	// confirm-delete
			switch_option(3, confirm_delete);
			break;
                case '4':
                        switch_option(4, skip_system_dialogs);
                        break;
		}
		print_general_options(conf_win);
	}
	delwin(conf_win);
}

/* prints the general options */
void print_general_options(WINDOW *win)
{
	int x_pos, y_pos;
	char option1[] = "auto_save = ";
	char option2[] = "confirm_quit = ";
	char option3[] = "confirm_delete = ";
        char option4[] = "skip_system_dialogs = ";

	x_pos = 3;
	y_pos = 4;

	mvwprintw(win, y_pos, x_pos, "[1] %s      ", option1);
	print_option_incolor(win, auto_save, y_pos,
			     x_pos + 4 + strlen(option1));
	mvwprintw(win, y_pos + 1, x_pos,
		 "(if set to YES, automatic save is done when quitting)");

	mvwprintw(win, y_pos + 3, x_pos, "[2] %s      ", option2);
	print_option_incolor(win, confirm_quit, y_pos + 3,
			     x_pos + 4 + strlen(option2));
	mvwprintw(win, y_pos + 4, x_pos,
		 "(if set to YES, confirmation is required before quitting)");

	mvwprintw(win, y_pos + 6, x_pos, "[3] %s      ", option3);
	print_option_incolor(win, confirm_delete, y_pos + 6,
			     x_pos + 4 + strlen(option3));
	mvwprintw(win, y_pos + 7, x_pos,
		 "(if set to YES, confirmation is required before deleting an event)");
        
	mvwprintw(win, y_pos + 9, x_pos, "[4] %s      ", option4);
	print_option_incolor(win, skip_system_dialogs, y_pos + 9,
			     x_pos + 4 + strlen(option4));
	mvwprintw(win, y_pos + 10, x_pos,
		 "(if set to YES, messages about loaded and saved data will not be displayed)");

	wnoutrefresh(win);
	doupdate();
}

/* print the option with appropriate color */
void print_option_incolor(WINDOW *win, char *option, int pos_y, int pos_x)
{
	int color;

	if (strncmp(option, "yes", 3) == 0)
		color = 2;
	else if (strncmp(option, "no", 2) == 0)
		color = 1;
	else {
		erase_window_part(win, 0, 0, col, row - 2);
		mvwprintw(win, 1, 1,
			 "option not defined - Problem in print_option_incolor()");
		wnoutrefresh(win);
		doupdate();
		wgetch(win);
		exit(1);
	}
	wattron(win, COLOR_PAIR(color));
	mvwprintw(win, pos_y, pos_x, "%s", option);
	wattroff(win, COLOR_PAIR(color));
	wnoutrefresh(win);
	doupdate();
}

  /* Change an option value */
void switch_option(int option, char *opt)
{
	if (strncmp(opt, "yes", 3) == 0) {
		strcpy(opt, "no ");
	} else if (strncmp(opt, "no", 2) == 0) {
		strcpy(opt, "yes");
	}
	if (option == 1)
		strcpy(auto_save, opt);
	if (option == 2)
		strcpy(confirm_quit, opt);
	if (option == 3)
		strcpy(confirm_delete, opt);
        if (option == 4)
                strcpy(skip_system_dialogs, opt);
}

  /* Delete an event from the ToDo or Appointment lists */
void del_event()
{
	char really_delete[3] = "ini";
	char del_app_str[] = "Do you really want to delete this Appointment ? [yes or no]";
	char del_todo_str[] = "Do you really want to delete this ToDo ? [yes or no]";
	long date;

	if (which_pan == 1 && hilt_app != 0) {	//appointment event to delete
		date = date2sec(sel_year, sel_month, sel_day, 0, 0);
		if (strncmp(confirm_delete, "yes", 3) == 0) {
			while ((strncmp(really_delete, "yes", 3) != 0) &
			       (strncmp(really_delete, "no", 3) != 0)) {
			        status_mesg(del_app_str, "", colr);					 
				getstring(swin, colr, really_delete, 0, 1);
			}
			if (strncmp(really_delete, "yes", 3) == 0) {
				if (number_events_inday != 0) {
					event_delete_bynum(date, hilt_app - 1);	//todo number to delete
					--number_events_inday;
				}
			} else {
				erase_window_part(swin, 0, 0, nc_bar, nl_bar);
				strcpy(really_delete, "ini");
				return;
			}
		} else {
			if (number_events_inday != 0) {
				event_delete_bynum(date, hilt_app - 1);
				--number_events_inday;
			}
		}

	} else if (which_pan == 2 && hilt_tod != 0) {	// todo event to delete
		if (strncmp(confirm_delete, "yes", 3) == 0) {
			while ((strncmp(really_delete, "yes", 3) != 0) &
			       (strncmp(really_delete, "no", 3) != 0)) {
			        status_mesg(del_todo_str, "", colr);
				getstring(swin, colr, really_delete, 0, 1);
			}
			if (strncmp(really_delete, "yes", 3) == 0) {
				if (nb_tod > 0) {
					todo_delete_bynum(hilt_tod - 1);
					//todo number to delete
					--nb_tod;
				}
			} else {
				erase_window_part(swin, 0, 0, nc_bar, nl_bar);
				strcpy(really_delete, "ini");
				return;
			}
		} else {
			if (nb_tod > 0) {
				todo_delete_bynum(hilt_tod - 1);
				//todo number to delete
				--nb_tod;
			}
		}
	}
}

  /* Adds an item in the ToDo list */
void add_todo()
{
	char mesg[] = "Enter the new ToDo event : ";
	char todo_input[500];

	status_mesg(mesg, "", colr);
	getstring(swin, colr, todo_input, 0, 1);
	if (strlen(todo_input) != 0) {
		todo_insert(todo_input);
		++nb_tod;
		update_todo_panel();
	}
	erase_window_part(swin, 0, 0, nc_bar, nl_bar);
	status_bar(which_pan, colr, nc_bar, nl_bar);
	doupdate();
}

  /* Adds An Item In The Appointments List */
void add_apts()
{
	char mesg_1[] = "Enter the start time [hh:mm] : ";
	char mesg_2[] = "Enter the event duration [in minutes] : ";
	char mesg_3[] = "Enter the event description :";
	char format_message_1[] = "You entered an invalid start time";
	char format_message_2[] = "Please enter the event duration in minutes";
        char enter_str[] = "Press [Enter] to continue";
	int i;
	char event_time[500];
	char event_mesg[500];
	long event_duration;
	struct event_s *event_pointeur;
	unsigned heures, minutes, duration;

	// get the starting time
	strcpy(event_time, "     ");
	while (check_event_time(event_time) == 0) {
                status_mesg(mesg_1, "", colr);
		getstring(swin, colr, event_time, 0, 1);
		if (strlen(event_time) == 0)
			return;	//nothing entered, cancel event add
		if (check_event_time(event_time) == 0) {
                        status_mesg(format_message_1, enter_str, colr);
			wgetch(swin);
		} else
			sscanf(event_time, "%u:%u", &heures, &minutes);
	}
	// get the event duration
	strcpy(event_time, " ");
	while (is_all_digit(event_time) == 0) {
                status_mesg(mesg_2, "", colr);
		getstring(swin, colr, event_time, 0, 1);
		if (strlen(event_time) == 0)
			return;	//if nothing entered, cancel event add
		if (is_all_digit(event_time) == 0) {
                        status_mesg(format_message_2, enter_str, colr);
			wgetch(swin);
		} else
			event_duration = atoi(event_time);
	}
	// get the appointment description
        status_mesg(mesg_3, "", colr);
	getstring(swin, colr, event_mesg, 0, 1);
	if (strlen(event_mesg) != 0) {
		// insert the appointment in list
		event_pointeur =
		    event_new(event_mesg,
			      date2sec(sel_year, sel_month, sel_day,
				       heures, minutes),
			      min2sec(event_duration));
		// update the panel
		update_app_panel();
	}
	erase_window_part(swin, 0, 0, nc_bar, nl_bar);
	status_bar(which_pan, colr, nc_bar, nl_bar);
	doupdate();
}

/* Updates the ToDo panel */
void update_todo_panel()
{
	WINDOW *popup_win;
	struct todo_s *i;
	int len = nc_tod - 8;
	char buf[len];
	int num;
	int y_offset;
	int title_lines = 3;

	num = 0;
	y_offset = 0;
	erase_window_part(twin, 1, title_lines, nc_tod - 2, nl_tod - 2);
	
	if (first_todo_onscreen > 1) {
		wattron(twin, COLOR_PAIR(colr));
		mvwprintw(twin, 3, 1, "^");
		wattroff(twin, COLOR_PAIR(colr));
	}
	for (i = todolist; i != 0; i = i->next) {
		num++;
		y_offset = 3 + num - first_todo_onscreen;
		if (num >= first_todo_onscreen) {	// checks the first item to print
			if (num - first_todo_onscreen < nl_tod - 4) {	//checks the available line space
				if (show_event == 0) {	// checks if normal or popup mode
					if (num == hilt_tod)
						wattron(twin, COLOR_PAIR(colr));	//checks the selected item
					if (strlen(i->mesg) < len)
						mvwprintw(twin, y_offset,
							 2, "- %s",
							 i->mesg);
					else {
						strncpy(buf, i->mesg,
							len - 1);
						buf[len - 1] = '\0';
						mvwprintw(twin, y_offset,
							 2,
							 "- %s...", buf);
					}
					if (num == hilt_tod)
						wattroff(twin, COLOR_PAIR(colr));
				} else {	//popup a window with the selected item
					if (num == hilt_tod) {
						popup_win =
						    popup(row - 4, col - 2,
							  1, 1, "To Do :",
							  colr);
						scroller(popup_win, i->mesg, 1, 1,
							 row - 4, col - 2,
							 colr);
                                                wmove(swin, 0, 0);
                                                doupdate();
						wgetch(popup_win);
						delwin(popup_win);
					}
				}	//end if normal or popup mode
			}
			if (num - first_todo_onscreen >= nl_tod - 4) {	//prints "v" at end of display if too many items
				wattron(twin, COLOR_PAIR(colr));
				mvwprintw(twin, nl_tod - 2, 1,
					 "v");
				wattroff(twin, COLOR_PAIR(colr));
			}
		}	
	}	
	wnoutrefresh(twin);
}

  /* Updates the Appointment panel */
void update_app_panel()
{
	WINDOW *popup_win;
	int j, pos_x;
	struct event_s *i;
	long date_start;
	int len = nc_app - 8;
	char buf[len];
	int max_events = floor((nl_app - 4) / 3);
	int y_offset;
	int title_lines = 3;

	number_events_inday = 0;
	y_offset = 0;
	erase_window_part(awin, 1, title_lines, nc_app - 2, nl_app - 2);
	
	// prints the selected date 
	pos_x = nc_app - (strlen(monthnames[sel_month - 1]) + 11);
	wattron(awin, COLOR_PAIR(colr));
	if (sel_day < 10)
		mvwprintw(awin, 3, pos_x, " %s %d, %d",
			 monthnames[sel_month - 1], sel_day, sel_year);
	else
		mvwprintw(awin, 3, pos_x, "%s %d, %d",
			 monthnames[sel_month - 1], sel_day, sel_year);
	wattroff(awin, COLOR_PAIR(colr));
// prints the appointments for selected day
	date_start = date2sec(sel_year, sel_month, sel_day, 0, 0);
	if (first_app_onscreen > 1) {
		wattron(awin, COLOR_PAIR(colr));
		mvwprintw(awin, 3, 1, "^");
		wattroff(awin, COLOR_PAIR(colr));
	}
	for (i = eventlist; i != 0; i = i->next) {
		if (event_inday(i, date_start)) {
			number_events_inday++;
			y_offset =
			    8 + 3 * (number_events_inday - 1) -
			    (first_app_onscreen * 3);
			event_sec2str(i, date_start, event_start_time,
				      event_end_time);
			if (number_events_inday >= first_app_onscreen) {
				if (number_events_inday -
				    first_app_onscreen < max_events) {
					if (show_event == 0)	//no popup mode
					{
						if (number_events_inday ==
						    hilt_app)
							wattron(awin, COLOR_PAIR
							       (colr));
						mvwprintw(awin, y_offset - 1, 1,
							 " - %s -> %s",
							 event_start_time,
							 event_end_time);
						//check message length
						if (strlen(i->mesg) < len)
							mvwprintw(awin, y_offset,
								 5,
								 "%s",
								 i->mesg);
						else {
							strncpy(buf,
								i->mesg,
								len - 1);
							buf[len - 1] =
							    '\0';
							mvwprintw(awin, y_offset,
								 5,
								 "%s...",
								 buf);
						}

						if (number_events_inday ==
						    hilt_app)
							wattroff(awin, COLOR_PAIR
								(colr));
					} else	//popup mode
					{
						if (number_events_inday ==
						    hilt_app) {
							popup_win =
							    popup(row - 4,
								  col - 2,
								  1, 1,
								  "Appointment :",
								  colr);
							mvwprintw(popup_win, 4, 4,
								 " - %s -> %s",
								 event_start_time,
								 event_end_time);
							scroller(popup_win, i->mesg,
								 1, 1,
								 row - 4,
								 col - 2,
								 colr);
                                                        wmove(swin, 0, 0);
                                                        doupdate();
							wgetch(popup_win);
							delwin(popup_win);
						}
					}	//end if normal or popup mode
				}
				if (number_events_inday -
				    first_app_onscreen >= max_events) {
					wattron(awin, COLOR_PAIR(colr));
					mvwprintw(awin, nl_app - 2,
						 1, "v");
					wattroff(awin, COLOR_PAIR(colr));
				}
			}
		}
	}
	wnoutrefresh(awin);
}

  /* Load the user configuration */
void load_conf()
{
	FILE *data_file;
	char mesg_line1[] = "Failed to open config file";
	char mesg_line2[] = "Press [ENTER] to continue";
	char buf[100], e_conf[100];
	int var;

	data_file = fopen(path_conf, "r");
	if (data_file == NULL) {
		status_mesg(mesg_line1, mesg_line2, colr);
                wnoutrefresh(swin);
                doupdate();
		wgetch(swin);
	}
	var = 0;
	for (;;) {
		if (fgets(buf, 99, data_file) == NULL) {
			break;
		}
		extract_data(e_conf, buf, strlen(buf));
		if (var == 1) {
			strcpy(auto_save, e_conf);
			var = 0;
		} else if (var == 2) {
			strcpy(confirm_quit, e_conf);
			var = 0;
		} else if (var == 3) {
			strcpy(confirm_delete, e_conf);
			var = 0;
		} else if (var == 4) {
			strcpy(skip_system_dialogs, e_conf);
			var = 0;
		} else if (var == 5) {
			colr = atoi(e_conf);
			var = 0;
		} else if (var == 6) {
			layout = atoi(e_conf);
			var = 0;
		}
		if (strncmp(e_conf, "auto_save=", 10) == 0)
			var = 1;
		else if (strncmp(e_conf, "confirm_quit=", 13) == 0)
			var = 2;
		else if (strncmp(e_conf, "confirm_delete=", 15) == 0)
			var = 3;
                else if (strncmp(e_conf, "skip_system_dialogs=", 20) == 0)
                        var = 4;
		else if (strncmp(e_conf, "color-theme=", 12) == 0)
			var = 5;
		else if (strncmp(e_conf, "layout=", 7) == 0)
			var = 6;
	}
	fclose(data_file);
	erase_window_part(swin, 0, 0, nc_bar, nl_bar);
}

  /* get data from file */
void extract_data(char *dst_data, const char *org, int len)
{
	for (;;) {
		if (*org == '\n' || *org == '\0')
			break;
		*dst_data++ = *org++;
	}
	*dst_data = '\0';
}
