/*
 * Calcurse - text-based organizer
 * Copyright (c) 2004-2005 Frederic Culot
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Send your feedback or comments to : calcurse@culot.org
 * Calcurse home page : http://culot.org/calcurse
 *
 */

#include <ncurses.h>
#include <time.h>	
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <math.h>

#include "utils.h"
#include "vars.h"


/* 
 * Print a message in the status bar.
 * Message texts for first line and second line are to be provided.
 */

void status_mesg(char *mesg_line1, char *mesg_line2, int color)
{
	erase_window_part(swin, 0, 0, col, 2);
	wattron(swin, COLOR_PAIR(color));
	mvwprintw(swin, 0, 0, mesg_line1);
	mvwprintw(swin, 1, 0, mesg_line2);
	wattroff(swin, COLOR_PAIR(color));
}

/* 
 * Erase part of a window 
 */
void erase_window_part(WINDOW *win, int first_col, int first_row,
			int last_col, int last_row)
{
	int c, r;

	for (r = first_row; r <= last_row; r++){
		for (c = first_col; c <= last_col; c++){
			mvwprintw(win, r, c, " ");
		}
	}
	wnoutrefresh(win);
}

/* draws a popup window */
WINDOW * popup(int pop_row, int pop_col,
	    int pop_y, int pop_x, char *pop_lab, int pop_colr)
{
	char txt_pop[] = "Press any key to continue...";
	char label[80];
	WINDOW *popup_win;

	popup_win = newwin(pop_row, pop_col, pop_y, pop_x);
	wattron(popup_win, COLOR_PAIR(pop_colr) | WA_BOLD);
	box(popup_win, 0, 0);
	sprintf(label, "%s", pop_lab);
	win_show(popup_win, label, pop_colr);
        wattroff(popup_win, WA_BOLD);
	wattron(popup_win, COLOR_PAIR(pop_colr));
	mvwprintw(popup_win, pop_row - 2, pop_col - (strlen(txt_pop) + 1), "%s",
		 txt_pop);
	wattroff(popup_win, COLOR_PAIR(pop_colr));
	wnoutrefresh(popup_win);
        doupdate();
	return popup_win;
}

/* prints in middle of a panel */
void
print_in_middle(WINDOW * win, int starty, int startx, int width,
		char *string, chtype color)
{
	int length, x, y;
	float temp;

	if (win == NULL)
		win = stdscr;
	getyx(win, y, x);
	if (startx != 0)
		x = startx;
	if (starty != 0)
		y = starty;
	if (width == 0)
		width = 80;

	length = strlen(string);
	temp = (width - length) / 2;
	x = startx + (int) temp;
	wattron(win, color);
	mvwprintw(win, y, x, "%s", string);
	wattroff(win, color);
}

/* 
 * Getstring allows to get user input and to print it on a window,
 * event if noecho() is on.
 */
void getstring(win, colr, string, start_x, start_y)
WINDOW *win;
int colr;
char *string;
int start_x, start_y;
{
	int ch;
	int charcount = 0;

	wattron(win, COLOR_PAIR(colr));
	if (start_x != -1)
		wmove(win, start_y, start_x);

	while ((ch = wgetch(win)) != '\n') {
	        if (ch == 263) {	/* backspace was hit */
			if (charcount > 0) {
				string--;
				charcount--;
				wmove(win, start_y, start_x + charcount);
				waddch(win, ' ');
				wmove(win, start_y, start_x + charcount);
			}
		} else {
			*string++ = ch;
			charcount++;
			waddch(win, ch);
		}
		doupdate();
	}
	*string = 0;
	wattroff(win, COLOR_PAIR(colr));
	return;
}

/* checks if a string is only made of digits */
int is_all_digit(char *string)
{
	int digit, i;
	int all_digit;

	digit = 0;
	all_digit = 0;

	for (i = 0; i <= strlen(string); i++)
		if (isdigit(string[i]) != 0)
			digit++;
	if (digit == strlen(string))
		all_digit = 1;
	return all_digit;
}

/* draw panel border in color */
void border_color(WINDOW * window, int bcolr)
{
	wattron(window, COLOR_PAIR(bcolr) | WA_BOLD);
	box(window, 0, 0);
	wattroff(window, COLOR_PAIR(bcolr) | WA_BOLD);
	wnoutrefresh(window);
}

 /* prints and scroll text in a window */
void scroller(WINDOW *win, char *mesg, int x, int y, int nb_row, int nb_col, int colr)
{
	int x_offset = 3;
	int y_offset = 3;
	int text_len = nb_col - 2 * x_offset;
	int text_max_row = nb_row - 3;
	int nlin, i, j, k;
	int last_blank_i, last_blank_j;
	char buf[] = " ";
	char *dst;
	char next_mesg[] = "-- Press 'N' for next page --";
	char prev_mesg[] = "-- Press 'P' for previous page --";
	int ch;
	int which_page;		//first page : 0, second page : 1

	i = 0;			//position in the message
	j = 0;			//x position on the current line
	nlin = 1;		//line number 
	last_blank_j = 0;
	last_blank_i = 0;
	which_page = 0;

	while (i <= strlen(mesg)) {
                if ((i == strlen(mesg)) & (which_page == 1)) {
			// we have finished writing text and we are on second page
			wattron(win, COLOR_PAIR(colr));
			mvwprintw(win, nb_row - 2,
				 nb_col - (strlen(prev_mesg) + 2), "%s",
				 prev_mesg);
			wattroff(win, COLOR_PAIR(colr));
                        wmove(swin, 0, 0);
			wnoutrefresh(win);
                        wnoutrefresh(swin);
                        doupdate();
			ch = wgetch(win);
			if (ch == 'P' | ch == 'p') {
				erase_window_part(win, y + 1, x + 3, nb_col - 2, nb_row - 2);
				nlin = 1;
				j = 0;
				i = 0;
				which_page = 0;
			} else {	//erase last line and exit next-prev page mode
				for (k = 1; k < nb_col - 2; k++)
					mvwprintw(win, nb_row - 2, k, " ");
				break;
			}
		}
		if (nlin == text_max_row - 2) {	// we reach the last line
			wattron(win, COLOR_PAIR(colr));
			mvwprintw(win, nb_row - 2,
				 nb_col - (strlen(next_mesg) + 2), "%s",
				 next_mesg);
			wattroff(win, COLOR_PAIR(colr));
                        wmove(swin, 0, 0);
			wnoutrefresh(win);
                        wnoutrefresh(swin);
                        doupdate();
			ch = wgetch(win);
			if (ch == 'N' | ch == 'n') {
				erase_window_part(win, y + 1, x + 3, nb_col - 2, nb_row - 2);
				nlin = 1;
				j = 0;
				which_page = 1;
			} else {
				for (k = 1; k < nb_col - 2; k++)
					mvwprintw(win, nb_row - 2, k, " ");
				break;
			}
		}
		//write text
		strncpy(buf, mesg + i, 1);
		i++;
		j++;
		if ((strncmp(buf, "", 1) == 0)) {	// is the character for a new line
			buf[0] = '\0';
			mvwprintw(win, x + x_offset + nlin, y + y_offset + j,
				 "%s", buf);
			nlin++;
			j = 0;
		} else {
			if (j == text_len - 1) {	// if we reach the terminal border
				for (k = last_blank_j; k <= text_len - 1;
				     k++)
					mvwprintw(win, x + x_offset + nlin,
						 y + y_offset + k, " ");
				nlin++;
				i = last_blank_i;
				j = 0;
			} else {
				if ((strncmp(buf, " ", 1) == 0))	//space between words
				{
					last_blank_j = j;	//save position
					last_blank_i = i;
				}
				mvwprintw(win, x + x_offset + nlin,
					 y + y_offset + j, "%s", buf);
			}
		}
        }
        wmove(swin, 0, 0);
        wnoutrefresh(win);
        wnoutrefresh(swin);
}

/* Draws the status bar */
void status_bar(int which_pan, int colr, int nc_bar, int nl_bar)
{
	int nb_item_cal, nb_item_oth;
	int len_let, len_des, spc_lad;
	int spc_bet_cal_itm, spc_bet_oth_itm;
	int len_cal_itm, len_oth_itm;

	nb_item_cal = 10;	/* max item number to display in status bar */
	nb_item_cal = ceil(nb_item_cal / 2);	/* two lines to display items */
	nb_item_oth = 12;
	nb_item_oth = ceil(nb_item_oth / 2);
	len_let = 3;
	len_des = 8;
	spc_lad = 1;

	spc_bet_cal_itm =
	    floor((col -
		   nb_item_cal * (len_let + len_des +
				  spc_lad)) / nb_item_cal);
	spc_bet_oth_itm =
	    floor((col -
		   nb_item_oth * (len_let + len_des +
				  spc_lad)) / nb_item_oth);
	len_cal_itm = len_let + spc_lad + len_des + spc_bet_cal_itm;
	len_oth_itm = len_let + spc_lad + len_des + spc_bet_oth_itm;

	erase_window_part(swin, 0, 0, nc_bar, nl_bar);
	if (which_pan == 0) {
		wattron(swin, COLOR_PAIR(colr));
		mvwprintw(swin, 0, 0, "  ?");
		mvwprintw(swin, 1, 0, "  Q");
		mvwprintw(swin, 0, len_cal_itm, "  R");
		mvwprintw(swin, 1, len_cal_itm, "  S");
		mvwprintw(swin, 0, 2 * len_cal_itm, "H/L");
		mvwprintw(swin, 1, 2 * len_cal_itm, "J/K");
		mvwprintw(swin, 0, 3 * len_cal_itm, "  G");
		mvwprintw(swin, 1, 3 * len_cal_itm, "Tab");
		mvwprintw(swin, 0, 4 * len_cal_itm, "  C");
		wattroff(swin, COLOR_PAIR(colr));
		wnoutrefresh(swin);

		mvwprintw(swin, 0, len_let + spc_lad, "Help");
		mvwprintw(swin, 1, len_let + spc_lad, "Quit");
		mvwprintw(swin, 0, len_cal_itm + len_let + spc_lad,
			 "Redraw");
		mvwprintw(swin, 1, len_cal_itm + len_let + spc_lad, "Save");
		mvwprintw(swin, 0, 2 * len_cal_itm + len_let + spc_lad,
			 "-/+1 Day");
		mvwprintw(swin, 1, 2 * len_cal_itm + len_let + spc_lad,
			 "-/+1 Week");
		mvwprintw(swin, 0, 3 * len_cal_itm + len_let + spc_lad,
			 "GoTo");
		mvwprintw(swin, 1, 3 * len_cal_itm + len_let + spc_lad,
			 "Chg View");
		mvwprintw(swin, 0, 4 * len_cal_itm + len_let + spc_lad,
			 "Config");
	} else {
		wattron(swin, COLOR_PAIR(colr));
		mvwprintw(swin, 0, 0, "  ?");
		mvwprintw(swin, 1, 0, "  Q");
		mvwprintw(swin, 0, len_oth_itm, "  R");
		mvwprintw(swin, 1, len_oth_itm, "  S");
		mvwprintw(swin, 0, 2 * len_oth_itm, "J/K");
		mvwprintw(swin, 1, 2 * len_oth_itm, "Tab");
		mvwprintw(swin, 0, 3 * len_oth_itm, "  A");
		mvwprintw(swin, 1, 3 * len_oth_itm, "  D");
		mvwprintw(swin, 0, 4 * len_oth_itm, "  G");
		mvwprintw(swin, 1, 4 * len_oth_itm, "  V");
		mvwprintw(swin, 0, 5 * len_oth_itm, "  C");
		wattroff(swin, COLOR_PAIR(colr));
		wnoutrefresh(swin);

		mvwprintw(swin, 0, len_let + spc_lad, "Help");
		mvwprintw(swin, 1, len_let + spc_lad, "Quit");
		mvwprintw(swin, 0, len_oth_itm + len_let + spc_lad,
			 "Redraw");
		mvwprintw(swin, 1, len_oth_itm + len_let + spc_lad, "Save");
		mvwprintw(swin, 0, 2 * len_oth_itm + len_let + spc_lad,
			 "Up/Down");
		mvwprintw(swin, 1, 2 * len_oth_itm + len_let + spc_lad,
			 "Chg View");
		mvwprintw(swin, 0, 3 * len_oth_itm + len_let + spc_lad,
			 "Add Item");
		mvwprintw(swin, 1, 3 * len_oth_itm + len_let + spc_lad,
			 "Del Item");
		mvwprintw(swin, 0, 4 * len_oth_itm + len_let + spc_lad,
			 "GoTo");
		mvwprintw(swin, 1, 4 * len_oth_itm + len_let + spc_lad,
			 "View");
		mvwprintw(swin, 0, 5 * len_oth_itm + len_let + spc_lad,
			 "Config");
	}
	wnoutrefresh(swin);
}
